﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using DotNetNuke.Entities.Modules;
using iFinity.DNN.Modules.CatalookUrlProvider.Entities;

namespace iFinity.DNN.Modules.CatalookUrlProvider.Data
{
    internal abstract class DataProvider
    {
        #region Shared/Static Methods

        // singleton reference to the instantiated object 
        static DataProvider objProvider = null;

        // constructor
        static DataProvider()
        {
            CreateProvider();
        }

        // dynamically create provider
        private static void CreateProvider()
        {
            //don't need run-time instancing when using built-in sqlDataProvider class
            objProvider = new Data.SqlDataProvider();
        }

        // return the provider
        public static DataProvider Instance()
        {
            return objProvider;
        }

        #endregion

        #region abstract methods
        /// <summary>
        /// Returns list of Urls related to your module
        /// </summary>
        /// <param name="portalId">The portalId of the portal.</param>
        /// <param name="entries">out parameter returns a zero-n list of Catalook Urls</param>
        /// <param name="clOptions">options configured for the catalook provider</param>
        internal abstract void GetCatalookItemsForPortal(int portalId, CatalookUrlOptions clOptions, out FriendlyUrlInfoCol urls);
        internal abstract FriendlyUrlInfo GetItemUrl(int itemId, string itemType, int portalId, CatalookUrlOptions clOptions);
        internal abstract void GetCategoriesForTabs(int portalId, CatalookUrlOptions clOptions, out List<TabCategory> tabCategories);
     
        #endregion

    }
}
