﻿using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Entities.Portals;
using iFinity.DNN.Modules.UrlMaster;
using iFinity.DNN.Modules.UrlRedirectProvider.Entities;

namespace iFinity.DNN.Modules.UrlRedirectProvider
{
    internal static class UrlController
    {
        //keys used for cache entries for Urls and Querystrings
        private const string RedirectIndexKey = "ifty_UrlRedirects_Portal_{0}";
        private const string RegexIndexKey = "ifty_UrlRedirectRegex_Portal_{0}";

        /// <summary>
        /// Returns a friendly url index from the cache or database.
        /// </summary>
        /// <param name="tabId"></param>
        /// <param name="portalId"></param>
        /// <param name="UrlRedirectModuleProvider"></param>
        /// <param name="options"></param>
        /// <returns></returns>
        internal static void GetRedirectIndexes(int portalId, UrlRedirectProvider provider, FriendlyUrlOptions options, out Hashtable redirectIndex, out List<RedirectInfo> regexIndex)
        {
            string redirectCacheKey = GetRedirectIndexKeyForPortal(portalId);
            string regexCacheKey = GetRegexIndexKeyForPortal(portalId);
            redirectIndex = (Hashtable)DataCache.GetCache(redirectCacheKey);
            regexIndex = (List<RedirectInfo>)DataCache.GetCache(regexCacheKey);
            if (redirectIndex == null || regexIndex == null)
            {
                //build index for tab
                BuildUrlIndexes(portalId, provider, options, out redirectIndex, out regexIndex);
                StoreIndexes(redirectIndex, redirectCacheKey, regexIndex, regexCacheKey);
            }
        }


        private static void BuildUrlIndexes(int portalId, UrlRedirectProvider provider, FriendlyUrlOptions options, out Hashtable friendlyUrlIndex, out List<RedirectInfo> regexIndex)
        {
            friendlyUrlIndex = new Hashtable();
            regexIndex = new List<RedirectInfo>();
            //call database procedure to get list of redirects
            List<RedirectInfo> redirects = DotNetNuke.Common.Utilities.CBO.FillCollection<RedirectInfo>(Data.DataProvider.Instance().GetUrlRedirectsForPortal(portalId));
            
            if (redirects != null)
            {
                foreach (RedirectInfo redirect in redirects)
                {
                    if (redirect.IsRegex == false)
                    {
                        string url = redirect.MatchUrl.ToLower();
                        friendlyUrlIndex.Add(url, redirect);
                    }
                    else
                    {
                        regexIndex.Add(redirect);
                    }
                }
            }
        }
        /// <summary>
        /// REbuilds the two indexes and re-stores them into the cache
        /// </summary>
        /// <param name="tabId"></param>
        /// <param name="portalId"></param>
        /// <param name="provider"></param>
        /// <param name="options"></param>
        private static void RebuildIndexes(int portalId, UrlRedirectProvider provider, FriendlyUrlOptions options)
        {
            Hashtable redirectIndex = null; List<RedirectInfo> regexIndex = null;
            string redirectCacheKey = GetRedirectIndexKeyForPortal(portalId);
            string regexCacheKey = GetRegexIndexKeyForPortal(portalId);
            //build index for tabs
            BuildUrlIndexes(portalId, provider, options, out redirectIndex, out regexIndex);
            StoreIndexes(redirectIndex, redirectCacheKey, regexIndex, regexCacheKey);
        }
        /// <summary>
        /// Store the two indexes into the cache
        /// </summary>
        /// <param name="redirectIndex"></param>
        /// <param name="redirectCacheKey"></param>
        /// <param name="queryStringIndex"></param>
        /// <param name="queryStringCacheKey"></param>
        private static void StoreIndexes(Hashtable redirectIndex, string redirectCacheKey, List<RedirectInfo> regexIndex, string regexCacheKey)
        {
            TimeSpan expire = new TimeSpan(24, 0, 0);
            DataCache.SetCache(redirectCacheKey, redirectIndex, expire);
        }

        /// <summary>
        /// Return the cache key
        /// </summary>
        /// <param name="tabId"></param>
        /// <returns></returns>
        private static string GetRedirectIndexKeyForPortal(int tabId)
        {
            return string.Format(RedirectIndexKey, tabId);
        }
        private static string GetRegexIndexKeyForPortal(int tabId)
        {
            return string.Format(RegexIndexKey, tabId);
        }
        /// <summary>
        /// Removes the scheme from the Url and returns as a stirng
        /// </summary>
        /// <param name="uri"></param>
        /// <returns></returns>
        internal static string StripSchemeAndQueryStringFromUrl(Uri uri)
        {
            string url = uri.AbsoluteUri;
            if (uri != null)
            {
                //remove hte scheme
                string scheme = uri.Scheme + Uri.SchemeDelimiter;
                if (url != null && url.StartsWith(scheme) && url.Length > scheme.Length)
                    url = url.Substring(scheme.Length);
                //remove the querystring
                if (string.IsNullOrEmpty(uri.Query) == false)
                    url = url.Substring(0, url.Length - uri.Query.Length);
            }
            return url;
        }
        internal static string StripSchemeFromUrl(Uri uri)
        {
            string url = uri.AbsoluteUri;
            if (uri != null)
            {
                string scheme = uri.Scheme + Uri.SchemeDelimiter;
                if (url != null && url.StartsWith(scheme) && url.Length > scheme.Length)
                    url = url.Substring(scheme.Length);
            }
            return url;
        }

        internal static string StripAliasFromUrl(string absoluteUrl, string httpAlias)
        {
            string url = "";
            if (string.IsNullOrEmpty(absoluteUrl) == false 
             && string.IsNullOrEmpty(httpAlias) == false 
             && httpAlias.Length < absoluteUrl.Length)
            {
                if (absoluteUrl.StartsWith(httpAlias))
                    url = absoluteUrl.Substring(httpAlias.Length);
            }
            return url;
        }
        /// <summary>
        /// Given a rule, returns a redirect location
        /// </summary>
        /// <param name="redirectInfo"></param>
        /// <param name="redirectUrl"></param>
        /// <returns></returns>
        internal static string RedirectLocation(RedirectInfo redirectInfo, string redirectUrl, string scheme, NameValueCollection queryString, ref List<string> messages)
        {
            string destinationUrl = "";
            switch (redirectInfo.DestType.ToLower())
            {
                case "tab":
                    string path = redirectInfo.DestUrl;
                    
                    if (redirectInfo.DestTabId > -1)
                    {
                        //construct the portal settings required for the destination
                        PortalAliasController pac = new PortalAliasController();
                        if (string.IsNullOrEmpty(redirectInfo.HttpAlias))
                            throw new NullReferenceException("Redirect HttpAlias must be specified for DestType of 'Tab'.  Redirect Id : " + redirectInfo.RedirectId.ToString());
                        PortalAliasInfo pa =  pac.GetPortalAlias(redirectInfo.HttpAlias, redirectInfo.PortalId);
                        PortalSettings ps = new PortalSettings(redirectInfo.DestTabId, pa);
                        if (string.IsNullOrEmpty(path) == false)
                        {
                            string qs;
                            SplitPathAndQuerystring(path, out path, out qs);
                            //now call down to the navigate url
                            destinationUrl = DotNetNuke.Common.Globals.NavigateURL(redirectInfo.DestTabId, ps, "", path);
                            if (queryString != null && queryString.Count > 0)
                                destinationUrl = AddQueryString(destinationUrl, queryString);
                            //check for querystring preservation
                            if (redirectInfo.KeepQueryString && qs != null)
                            {
                                //put the querystring back on, if it existed
                                destinationUrl = AddQueryString(destinationUrl, qs);
                                messages.Add("URP: Redirecting " + redirectUrl + " to " + destinationUrl + " by tab + path + qs. RuleId: "+  redirectInfo.RedirectId.ToString());
                            }
                            else
                                messages.Add("URP: Redirecting " + redirectUrl + " to " + destinationUrl + " by tab + path. RuleId: " + redirectInfo.RedirectId.ToString());
                        }
                        else
                        {
                            destinationUrl = DotNetNuke.Common.Globals.NavigateURL(redirectInfo.DestTabId, ps, "");
                            if (redirectInfo.KeepQueryString && queryString != null && queryString.Count > 0)
                            {
                                destinationUrl = AddQueryString(destinationUrl, queryString);
                                messages.Add("URP: Redirecting " + redirectUrl + " to " + destinationUrl + " by tab + qs. RuleId: " + redirectInfo.RedirectId.ToString());
                            }
                            else
                                messages.Add("URP: Redirecting " + redirectUrl + " to " + destinationUrl + " by tab. RuleId: " + redirectInfo.RedirectId.ToString());
                        }
                    }
                    break;
                case "url":
                    destinationUrl = redirectInfo.DestUrl;
                    if (Regex.IsMatch(destinationUrl, @"^https?://", RegexOptions.IgnoreCase | RegexOptions.CultureInvariant) == false)
                    {
                        //no scheme, add on supplied scheme
                        destinationUrl = scheme + Uri.SchemeDelimiter + destinationUrl;
                    }
                    //keep query string from redirect destination
                    if (redirectInfo.KeepQueryString)
                    {
                        destinationUrl = AddQueryString(destinationUrl, queryString);
                        messages.Add("URP: Redirecting " + redirectUrl + " to " + destinationUrl + " by url + qs. RuleId: " + redirectInfo.RedirectId.ToString());
                    }
                    else
                        messages.Add("URP: Redirecting " + redirectUrl + " to " + destinationUrl + " by url. RuleId: " + redirectInfo.RedirectId.ToString());
                    break;
            }
            return destinationUrl;
        }

        private static string AddQueryString(string url, NameValueCollection queryString)
        {
            bool first = true;
            if (url.Contains("?"))
                first = false;
            if (queryString != null)
            {
                foreach (string key in queryString.AllKeys)
                {
                    string value = queryString[key];
                    if (value == null) value = "";
                    if (first)
                        url += "?" + key + "=" + value;
                    else
                        url += "&" + key + "=" + value;
                    first = false;
                }
            }
            return url;
        }

        private static string AddQueryString(string url, string qs)
        {
            bool first = true;
            if (url.Contains("?"))
                first = false;
            if (qs.StartsWith("?") || qs.StartsWith("&"))
                if (qs.Length > 0)
                    qs = qs.Substring(1);

            if (first)
                url += "?" + qs;
            else
                url += "&" + qs;

            return url;
        }
        internal static void SplitPathAndQuerystring(string url, out string path, out string queryString)
        {
            path = url;
            queryString = null;
            if (string.IsNullOrEmpty(path))
            {
                if (path.Contains("?"))
                {
                    string[] bits = path.Split('?');
                    if (bits.GetUpperBound(0) >=0)
                    {
                        path = bits[0];
                        queryString = bits[1];
                    }
                }
            }
        }
        
        internal static void CheckRegexRule(RedirectInfo regexRedirect, string requestedUrl, string requestedScheme, NameValueCollection queryStringCol, out bool doRedirect, out string redirectLocation, ref List<string> messages)
        {
            doRedirect = false;
            redirectLocation = null;
            if (string.IsNullOrEmpty(regexRedirect.RedirectUrl) == false)
            {
                string pattern = regexRedirect.RedirectUrl;
                string replace = regexRedirect.DestUrl;
                Regex urlRegex = new Regex(pattern, RegexOptions.IgnoreCase | RegexOptions.CultureInvariant);
                if (urlRegex.IsMatch(requestedUrl))
                {
                    //matches
                    messages.Add("URP : Regex Match on " + requestedUrl + " with pattern " + pattern);
                    string replacedUrl = urlRegex.Replace(requestedUrl, replace);
                    redirectLocation = RedirectLocation(regexRedirect, requestedUrl, requestedScheme, queryStringCol, ref messages);
                    doRedirect = true;
               }
            }
            
        }
    }
}
